% ML_ReadDataEU - upload and transforms Euro Area quarterly Data
%
% [Data, codeData] = ML_ReadData(trans,out)
%   trans = 0 - No transformation except logarithms
%   trans = 1 - Light transformation
%   trans = 2 - Heavy transformation
%   out   = 1 - Remove outliers (Procedure by Mario Forni)
%   out   = 2 - Remove outliers (Procedure by Matteo Barigozzi)
%

% Original version by Mario Forni, 
% modified by Matteo Luciani - mluciani@ulb.ac.be

function [Data, label, TT, cd, ordine] = ML_ReadDataEU(trans,out)

DB=dlmread('DB_EU_b.txt');                                                  % Upload the Database
[Label ordine]=LabelVariables;                                              % Upload label
X=DB(8:size(DB,1),:); n=size(X,2);                                          % Select Observations
if trans==2; apt=3; else apt=2; end; Tsc = DB(apt,:);                       % Select the Type of Transformation

%%% ========================================= %%%
%%% == Selecting and Transforming Varables == %%%
%%% ========================================= %%%

% Selecting %
sel=DB(1,:); jj=1;
for ii = 1:n;
    if sel(:,ii)==1; 
        x(:,jj)=X(:,ii); 
        tsc(:,jj)=Tsc(:,ii); 
        label(jj,:)=Label(ii,:); 
        jj=jj+1;
    end;
end;
[T,N]=size(x);

% Transforming %
if trans==0; bb=1; t0=1983;
    for ii=1:N; 
        if tsc(:,ii)==2; tsc(:,ii)=1;
        elseif (tsc(:,ii)==3)||(tsc(:,ii)==4); tsc(:,ii)=5; 
        end; 
    end
elseif trans==1; bb=2; t0=1983.25; 
elseif trans==2; bb=3; t0=1983.5; 
end

TT=t0:.25:2008.75; 

for i=1:N;
    if      tsc(i)==1;  data(:,i)=x(bb:T,i);                                cd(i)=0;
    elseif  tsc(i)==2;  data(:,i)=ML_diff(x(bb-1:T,i));                     cd(i)=1;
    elseif  tsc(i)==3;  data(:,i)=ML_diff(100*log(x(bb-1:T,i)));            cd(i)=1;
    elseif  tsc(i)==4;  data(:,i)=ML_diff(ML_diff(100*log(x(bb-2:T,i))));   cd(i)=2;
    elseif  tsc(i)==5;  data(:,i)=100*log(x(bb:T,i));                       cd(i)=0; end ;
end
 
if out==1;
    Data = removeoutliers(data);
elseif out==2;
    Data = removeoutliers2(data);
else
    Data=data;
end

            %%% ========================= %%%
            %%% == ------------------- == %%%
            %%% == - Remove Outliers - == %%%            
            %%% == ------------------- == %%%
            %%% ========================= %%%

% -------------------------------------------------------------------------------------
% Procedure |           Definition of outlier               |     Replaced with       |
% =====================================================================================
%   MF      | a values differing from the median more       | the median of the five  | 
%           | than 6 times the interquartile difference     | previous observations   |
% -------------------------------------------------------------------------------------
%   MB      |a  value greater than five standard deviations | the mean of the series  |
% -------------------------------------------------------------------------------------

%%% Codes by Mario Forni %%%
function cleaneddata = removeoutliers(data)
for i = 1:size(data,2);
    iqr_data = iqr(data(:,i)); % Interquantile Range
    amd_tdata = abs(data(:,i) - median(data(:,i))); % Distance from Median Value
    a  = find(amd_tdata >= 6*iqr_data); % Identify Outliers
    if ~isempty(a)
        for j = 1:length(a)
            if a(j)>1; data(a(j),i) = median(data(max(1,a(j)-5):a(j)-1,i));
            else         data(a(j),i) =  median(data(2:6, i)); end
        end
    end
end
cleaneddata = data;

%%% Codes by Matteo Barigozzi %%%
function cleaneddata = removeoutliers2(data)

y=ML_Standardize(data);
mu=mean(data);
for i = 1:size(data,2);
    a  = find(y(:,i) > 5);                                            % Identify Outliers
    if ~isempty(a) for j = 1:length(a); data(a(j),i)=mu(i); end; end; % Replace the Outlier
end
cleaneddata = data;
 
%%% ============================= %%%
%%% == - Labelling variables - == %%%
%%% ============================= %%%
function [label ordine]=LabelVariables
label= [
'BG.Y     ';'BG.C     ';'BG.I     ';'BG.G     ';'BG.X     ';'BG.M     ';'BG.Y.p   ';'BG.C.p   ';'BG.I.p   ';'BG.X.p   ';
'BG.M.p   ';'BG.cars  ';'BG.cuj   ';'BG.L     ';'BG.u     ';'BG.ulc   ';'BG.ulc.F ';'BG.ulc.D ';'BG.ulc.gk';'BG.ulc.jk';
'BG.reer  ';'BG.retail';'BG.ppi   ';'BG.cpi   ';'BG.cpi.fe';            'BG.cpi.e ';            'BG.str   ';'BG.ltr   ';
'BG.share ';'BG.w     ';'BG.M1    ';            'BG.M3    ';
'FR.Y     ';'FR.C     ';'FR.I     ';'FR.G     ';'FR.X     ';'FR.M     ';'FR.Y.p   ';'FR.C.p   ';'FR.I.p   ';'FR.X.p   ';
'FR.M.p   ';'FR.cars  ';'FR.cuj   ';'FR.L     ';'FR.u     ';'FR.ulc   ';'FR.ulc.F ';'FR.ulc.D ';'FR.ulc.gk';'FR.ulc.jk';
'FR.reer  ';'FR.retail';'FR.ppi   ';'FR.cpi   ';'FR.cpi.fe';'FR.cpi.f ';'FR.cpi.e ';'FR.cpi.r ';'FR.str   ';'FR.ltr   ';
'FR.share ';'FR.w     ';'FR.M1    ';'FR.M2    ';'FR.M3    ';
'GE.Y     ';'GE.C     ';'GE.I     ';'GE.G     ';'GE.X     ';'GE.M     ';'GE.Y.p   ';'GE.C.p   ';'GE.I.p   ';'GE.X.p   ';
'GE.M.p   ';'GE.cars  ';'GE.cuj   ';'GE.L     ';'GE.u     ';'GE.ulc   ';'GE.ulc.F ';'GE.ulc.D ';'GE.ulc.gk';'GE.ulc.jk';
'GE.reer  ';'GE.retail';'GE.ppi   ';'GE.cpi   ';'GE.cpi.fe';'GE.cpi.f ';'GE.cpi.e ';'GE.cpi.r ';'GE.str   ';'GE.ltr   ';
'GE.share ';'GE.w     ';'GE.M1    ';'GE.M2    ';'GE.M3    ';
'IT.Y     ';'IT.C     ';'IT.I     ';'IT.G     ';'IT.X     ';'IT.M     ';'IT.Y.p   ';'IT.C.p   ';'IT.I.p   ';'IT.X.p   ';
'IT.M.p   ';'IT.cars  ';'IT.cuj   ';'IT.L     ';'IT.u     ';'IT.ulc   ';'IT.ulc.F ';'IT.ulc.D ';'IT.ulc.gk';'IT.ulc.jk';
'IT.reer  ';'IT.retail';'IT.ppi   ';'IT.cpi   ';'IT.cpi.fe';'IT.cpi.f ';'IT.cpi.e ';'IT.cpi.r ';'IT.str   ';'IT.ltr   ';
'IT.share ';'IT.w     ';'IT.M1    ';'IT.M2    ';'IT.M3    ';
'NL.Y     ';'NL.C     ';'NL.I     ';'NL.G     ';'NL.X     ';'NL.M     ';'NL.Y.p   ';'NL.C.p   ';'NL.I.p   ';'NL.X.p   ';
'NL.M.p   ';'NL.cars  ';'NL.cuj   ';'NL.L     ';'NL.u     ';'NL.ulc   ';'NL.ulc.F ';'NL.ulc.D ';'NL.ulc.gk';'NL.ulc.jk';
'NL.reer  ';'NL.retail';'NL.ppi   ';'NL.cpi   ';'NL.cpi.fe';'NL.cpi.f ';'NL.cpi.e ';'NL.cpi.r ';'NL.str   ';'NL.ltr   ';
'NL.share ';'NL.w     ';'NL.M1    ';'NL.M2    ';'NL.M3    ';
'ES.Y     ';'ES.C     ';'ES.I     ';'ES.G     ';'ES.X     ';'ES.M     ';'ES.Y.p   ';'ES.C.p   ';'ES.I.p   ';'ES.X.p   ';
'ES.M.p   ';'ES.cars  ';'ES.cuj   ';'ES.L     ';'ES.u     ';'ES.ulc   ';'ES.ulc.F ';'ES.ulc.D ';'ES.ulc.gk';'ES.ulc.jk';
'ES.reer  ';            'ES.ppi   ';'ES.cpi   ';'ES.cpi.fe';'ES.cpi.f ';'ES.cpi.e ';'ES.cpi.r ';'ES.str   ';'ES.ltr   ';
'ES.share ';'ES.w     ';'ES.M1    ';'ES.M2    ';'ES.M3    ';
'FI.Y     ';'FI.cpi   ';'FI.str   ';'FI.ltr   ';
'GR.Y     ';'GR.cpi   ';'GR.str   ';
'IE.Y     ';'IE.cpi   ';'IE.str   ';'IE.ltr   ';
'LX.Y     ';'LX.cpi   ';
'PT.Y     ';'PT.cpi   ';'PT.str   ';'PT.ltr   ';
'UK.Y     ';'UK.cpi   ';'UK.str   ';'UK.ltr   ';
'JP.Y     ';'JP.cpi   ';'JP.str   ';'JP.ltr   ';
'US.Y     ';'US.cpi   ';'US.str   ';'US.ltr   ';
'EA.Y     ';'EA.cpi   ';'EA.str   ';'EA.ltr   ';'EA.ulc   ';'EA.M1    ';'EA.M2    ';'EA.M3    ';'EA.eer   ';
'useu     ';'brent    ';'texas    '];

%% This is necessary when printing tables, it groups the variables not by
%% country but by economic concept. It works only when M2 is backed out
ordine=[1;31;65;99;132;166;198;202;205;209;211;215;219;223;227;
2;32;66;100;133;167;3;33;67;101;134;168;34;68;102;135;169;5;35;69;103;136;170;6;36;70;104;137;171;
7;37;71;105;172;8;38;72;106;139;9;39;73;107;140;174;10;40;74;108;141;175;11;41;75;109;142;176;
12;42;76;110;143;177;13;43;77;111;144;178;14;44;78;112;145;179;15;45;79;113;146;180;
16;46;80;114;147;181;231;17;47;81;115;148;182;18;48;82;116;149;183;19;49;83;117;150;184;20;50;84;118;151;185;
21;51;85;119;152;186;234;22;52;86;153;23;53;87;120;154;187;
24;54;88;121;155;188;199;203;206;210;212;216;220;224;228;
25;55;89;122;156;189;56;90;123;157;26;57;91;124;158;190;58;92;125;159;191;
27;59;93;126;160;192;200;204;207;213;217;221;225;229;
28;60;94;127;161;193;201;208;214;218;222;226;230;
29;61;95;128;194;30;62;96;129;163;195;
63;97;130;164;196;232;64;98;131;165;197;233;235];
