function varargout=DrawStates(StateVartt,SIGtt,G1,G2,T,nStateVar)

% CRDrawStates
%
% Runs Kalman Smoother and draws state conditional on Kalman Smoother.
%
% Usage:
%   StateVard = DrawStates(StateVartt,SIGtt,G1,G2,T)
%   [StateVartT,SIGtT] = DrawStates(StateVartt,SIGtt,G1,G2,T)
%   [StateVard,StateVartT,SIGtT] = DrawStates(StateVartt,SIGtt,G1,G2,T)
%
% See also:
% SetDSGE, GenSymVars, DataAnalysis, PriorAnalysis, GenPost, MaxPost,
% MaxPostFcn, MakeTableMaxPost, MCMC, MCMCFcn, MCMCSearchScaleFactor,
% MakePlotsMCMCDraws, MCMCInference, MakeTableMCMCInference, 
% MakePlotsMCMCTrace, MakePlotsMCMCPriorPost, MCMCConv, MakeTableMCMCConv,
% MakeReportMCMCPlots, MakeReportMCMC
%
% .........................................................................
% 
% Created: October 21, 2008
% Updated: September 18, 2009
% by Vasco Curdia

%% ------------------------------------------------------------------------

%% Run smoother and draw state
StateVartT = StateVartt';
SIGtT = SIGtt;
Om = G2*G2';
StateVard = StateVartT;
StateVard(T,:)=DrawState(StateVartT(T,:),SIGtT(:,:,T),nStateVar);
for t=T-1:-1:1
    [StateVartT(t,:),SIGtT(:,:,t)]=ksmooth(StateVartT(t,:),SIGtT(:,:,t),StateVartT(t+1,:),SIGtT(:,:,t+1),G1,Om);
    StateVard(t,:)=DrawState(StateVartT(t,:),SIGtT(:,:,t),nStateVar);
end

%% prepare output
if nargout==1
    varargout = {StateVard'};
elseif nargout==2
    varargout = {StateVartT',SIGtT};
else
    varargout = {StateVard',StateVartT',SIGtT};
end

%% ------------------------------------------------------------------------

function StateVard=DrawState(StateVartT,ptT,nStateVar)

% NOTE: the trick below is intended at rounding errors that could make the
%       matrix look like it is not semi-definite positive.

% Old method
% % NumPrecision = 1e-16;
% [uu,dd,vv]=svd(ptT);
% % [uu,dd]=eig(ptT);
% % dd=round(dd/NumPrecision)*NumPrecision;
% StateVard=StateVartT+mvnrnd(zeros(nStateVar,1),dd)*uu';

% ptT = vcCovDefPos(ptT);
% StateVard=mvnrnd(StateVartT,ptT);

[u,d,v]=svd(ptT);
StateVard=StateVartT+mvnrnd(zeros(nStateVar,1),d)*u';

%% ------------------------------------------------------------------------
