function MakeTableMaxPost(TableFilename,Params,Post)

% MakeTableMaxPost
%
% Creates a table in TeX that can be loaded to SWP or compiled with LaTeX.
%
% Optional settings:
%   
% See also:
% SetDSGE, GenSymVars, DataAnalysis, PriorAnalysis, GenPost, MaxPost,
% MaxPostFcn, MakeTableMaxPost, MCMC, MCMCFcn, MCMCSearchScaleFactor,
% MakePlotsMCMCDraws, MCMCInference, MakeTableMCMCInference, 
% MakePlotsMCMCTrace, MakePlotsMCMCPriorPost, MCMCConv, MakeTableMCMCConv,
% MakeReportMCMCPlots, MakeReportMCMC
%
% .........................................................................
%
% Created: March 26, 2008
% Updated: November 3, 2008
% by Vasco Curdia

%% ------------------------------------------------------------------------

%% preamble
np = length(Params);

%% Initiate file
fid=fopen([TableFilename,'.tex'],'wt');
fprintf(fid,'\n\\documentclass{article}\n');
fprintf(fid,'\\setlength{\\oddsidemargin}{0.0in}\n');
fprintf(fid,'\\setlength{\\evensidemargin}{0.0in}\n');
fprintf(fid,'\\setlength{\\topmargin}{-0.5cm}\n');
fprintf(fid,'\\setlength{\\textheight}{9in}\n');
fprintf(fid,'\\setlength{\\textwidth}{6.5in}\n');
fprintf(fid,'\\usepackage{fancyhdr}\n');
fprintf(fid,'\\renewcommand{\\headrulewidth}{0pt}\n');
fprintf(fid,'\\pagestyle{fancy}\n');
fprintf(fid,'\\fancyhf{}\n');
fprintf(fid,'\\fancyhead[C]{\\textsc{%s}}\n',TableFilename);
fprintf(fid,'\\begin{document}\n');

%% value of the posterior density
% fprintf(fid,'\\begin{center}\n');
fprintf(fid,'\\begin{eqnarray*} \n');
fprintf(fid,'\\begin{tabular}{rl} \n');
fprintf(fid,'posterior log density at mode: & %.4f\n',Post.ModePost);
fprintf(fid,'\\end{tabular}\n');
fprintf(fid,'\\end{eqnarray*}\n');
% fprintf(fid,'\\end{center}\n');

%% Begin table
fprintf(fid,'\\begin{eqnarray*} \n');
fprintf(fid,'\\begin{tabular}{cccccccccccc} \n');
fprintf(fid,'\\hline\\hline\n');
fprintf(fid,'& & \\multicolumn{7}{c}{Prior} & & \\multicolumn{2}{c}{Posterior} \\\\\n');
fprintf(fid,'& & Dist & Mode & Mean & SE & 5\\%% & Median & 95\\%% & & Mode & SE \\\\\n');
fprintf(fid,'\\hline\n');
% fprintf(fid,'& & & & & & & & & & & \\\\\n');

%% Body of table
DispList = {'priormode','priormean','priorse','priorp050','priorp500',...
    'priorp950','postmode','postse'};
nc = length(DispList);
idxPost = find(ismember(DispList,'postmode'));
for jr=1:np
    str2show = ['$',strrep(Params(jr).prettyname,'\','\\'),'$'];
    str2show = [str2show,' & & ',Params(jr).priordist];
    for jc=1:nc
        if jc==idxPost,str2show = [str2show,' &'];end
        str2show = sprintf('%s & %.4f',str2show,Params(jr).(DispList{jc}));
    end
        str2show=[str2show,' \\\\\n'];
    fprintf(fid,str2show);
end

%% finish table
% fprintf(fid,'\\\\ \\multicolumn{12}{l}{} \\\\ \n');
fprintf(fid,'\\hline\\hline\n');
% fprintf(fid,['\\multicolumn{12}{l}{* For the igam ',...
%     'distribution, the shape parameter is set to 2 when SE is $\\infty$.} \n']);
fprintf(fid,'\\end{tabular} \n');
fprintf(fid,'\\end{eqnarray*} \n');

%% finish document and close file
fprintf(fid,'\\end{document}\n');
fclose(fid);

%% ------------------------------------------------------------------------
