function Out=MaxPostFcn(PostFileName,Params,MinParams)

% MaxPostFcn
%
% Maximizes the posterior by minimizing the negative of it.
%
% Inputs: [REQUIRED]
%   
%   PostFileName
%   Name of posterior evaluation function
%
%   Params
%   Params structure
%
% See also:
% SetDSGE, GenSymVars, DataAnalysis, PriorAnalysis, GenPost, MaxPost,
% MaxPostFcn, MakeTableMaxPost, MCMC, MCMCFcn, MCMCSearchScaleFactor,
% MakePlotsMCMCDraws, MCMCInference, MakeTableMCMCInference, 
% MakePlotsMCMCTrace, MakePlotsMCMCPriorPost, MCMCConv, MakeTableMCMCConv,
% MakeReportMCMCPlots, MakeReportMCMC
%
% .........................................................................
%
% Created: March 25, 2008
% Updated: August 24, 2010
% by Vasco Curdia

%% ------------------------------------------------------------------------

%% reset random number generator
s = clock;s = (s(6)-round(s(6)))*100000;rand('state',s);
s = clock;s = (s(6)-round(s(6)))*100000;randn('state',s);

%% default options
nMaxDraws = 1000;
UsePrior = 0;
x0Mean = [Params(:).priormean]';
x0SE = [Params(:).priorse];
x0df = 4;

%% Update options
if exist('MinParams','var') && isfield(MinParams,'Guess')
    GuessFields = fieldnames(MinParams.Guess);
    for jf=1:length(GuessFields)
        eval([GuessFields{jf},'=MinParams.Guess.',GuessFields{jf},';'])
    end
    MinParams = rmfield(MinParams,'Guess');
end

%% Guess values
np = length(Params);
if ~exist('x0','var')
    for jg=1:nMaxDraws
        if UsePrior
            disp('using prior')
            for j=1:np
                x0(j,1) = eval(Params(j).priorrndcmd);
            end
        else
            disp('using t-student')
            for jp=1:np
                for jpg=1:nMaxDraws*10
                    x0j = x0Mean(jp)+x0SE(jp)*trnd(x0df);
                    if strcmp(Params(jp).priordist,'N')
                        break
                    elseif strcmp(Params(jp).priordist,'B') && x0j>0 && x0j<1
                        break
                    elseif strcmp(Params(jp).priordist,'G') && x0j>0
                        break
                    elseif strcmp(Params(jp).priordist,'IG1') && x0j>0
                        break
                    elseif strcmp(Params(jp).priordist,'IG2') && x0j>0
                        break
                    end
                end
                x0(jp,1) = x0j;
            end
        end
        f0 = feval(PostFileName,x0);
        if f0>1e50
            fprintf('WARNING: Bad initial guess. Drawing a new one.\n')
        else
            break
        end
    end
end


%% initial inverse hessian
if ~exist('MinParams','var') || ~isfield(MinParams,'H0')
    H0 = [Params(:).priorse].^2;
    idx = (H0==inf); H0(idx) = [Params(idx).priormean];
    MinParams.H0 = diag(H0);
end

%% maximize the posterior
Out = vcRobustMin(PostFileName,x0,MinParams);

%% ------------------------------------------------------------------------
