  function [stt,etamat,smooth_st,yfor,vstar,logLnc,countszero]=kfilter_full(y,Z,T,R,Qmat,ins)
% function [stt,etamat,smooth_st,yfor,vstar,logLnc,countszero]=kfilter_full(y,Z,T,R,Qmat,ins)
% 
% Kalman Filtering Algortihm using the kf_mod.m routine  
% for non-diffuse models, when obs equation has no errors
% and all matrices except perhaps the volatilities are time invariant. 
% Notations follows Durbin and Koopman Ch 4 
%
% SAME AS KFILTER_MOD.M but store here as well the 
% smoothed states, forecasts and forecast errors 
% ====================================================================
% 
% Model 
% y(t)   = Z*s(t) 
% s(t)   = T*s(t-1) + R*n(t)    V( n(t) ) = Q(t)
% 
% Where y(t) is NZ x 1, x is NY x 1 and n is NX x 1 
% 
% Inputs 
% ======
% y                 nobs x NZ stacked vector of observations, nobs is the total number of observations  
%   
% flag_instate      If == 1 will fill in the initial states at indstate 
%                   with the initial observations and truncate the vector
%                   of ys'

% All matrices but Q(t) are asumed to be time invariant to simplify the computation 
% The correct size of input Qmat then depends on whether Q is time varyong or  not. 
% not time varying    Qmat is NX x NX 
%      time varying   Qmat is NX x NX x nobs
% 
%  NOTE! regardless of whether time variant or not, Qmat should be the Cholelsky of Q(t) 
%        such that Q(t)=Qmat(t)'*Qmat(t) 
%
% The code is intended to be used with a disturbance smoother rather than
%     with a standard and Kohn sampling algorithm 
% Suggess creating to other codes kfilter_CK where do not need to keep
%     track of Finv and Kgain 
% Also supress the states and Ptt here 
% For the case of no-variation, simplify the code by stoping the
%       computations once a fixed point has been found 
% 
% Check recover the observables  May 7 2007
%
% October 25 2007:Added PNX and COUNTSZERO 
% PNX decomposes the initial covariance matrix into the pieces coming from
% each ORTHOGONAL shock [NS NS NX]. This decomposition is valid only for 
% orthogonal elements & diagonal Qmat 
% COUNTSZERO is the initial state determined by each shock 
% Note that SUM(COUNTZERO) should equal SMOOTH_ST(:,1) 
% This is used in MODE_COUNTERCONT.m to ensure the counterfactual paths 
% of states and observables add up 
% ==================================================================
[nobs,nz]=size(y); 
[ny,nx] = size(R); 
ch=size( Qmat ); 
if ch(1) == nx; 
    flag_tv = 0 ;
    RQzero=R*Qmat'*Qmat*R'; 
elseif ch(1) == nobs;  
    flag_tv = 1; 
    temp=diag( Qmat(1,:) ); 
    RQzero=R*(temp'*temp)*R';
end 
clear temp; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization of the Filter 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ss=size(T,1);
nobs=size(y,1);
y=y'; 

% =============================================
% Obtain Initial states and Initial dispersion 
% =============================================
if nargin > 5 && ~isempty(ins); 
    flag_instate=1; 
    disp('Prior for initial state provided') 
    % 1. Initial states relating to observables 
    % ========================================
    azero=zeros(ss,1);
    if ~isempty(ins.state_data);
        disp('Filling initial state with data entries')
        tempz=Z';
        scalez=tempz(find(tempz~=0));
        clear tempz;
        if length(scalez)==size(Z,1)
            azero(ins.state_data)=y(:,1)./(scalez(:));
        end
        [ch11,ch21,ch31]=ss_var(T,R,Qmat);
        % LOC and NEWSTD will be defined ONLY if the dispersion of the
        % initial state is given by the user
        if ~isempty(ins.state_data_std)
            pshatloc=ins.state_data(:);newstd=ins.state_data_std(:);
        else
            pshatloc=[]; newstd=[];
        end
        y=y(:,2:end);nobs=nobs-1;
    else
        pshatloc=[];newstd=[];
    end
    % 2. Initial states for NON-observables 
    % ===================================
    if ~isempty(ins.state_loc);
        disp('Filing initial state with remaining entries');
        azero(ins.state_loc)=ins.state_mean;
        % LOC and NEWSTD will be defined ONLY if the dispersion of the
        % initial state is given by the user
        if ~isempty(ins.state_std) 
            pshatloc=[pshatloc;ins.state_loc(:)];newstd=[newstd(:);ins.state_std(:)];
        end 
        disp('Adjusting PSHAT');
    end
    pshat=lyapunov_symm(T,RQzero);
    if ~isempty(pshatloc) 
        pshat=vcv_rescale(pshat,pshatloc,newstd);
    end 
    pnx=[]; 
else 
    % Initial states and volatilities NOT modified
    flag_instate=0;
    azero=zeros(ss,1);
    pshat=disclyap_fast(T,RQzero);
    if ~isempty(find(isnan(pshat))~=0)
        error('PSHAT has NAN entries'); 
    end 
    pnx=zeros(ss,ss,nx);
    for ii=1:nx
        pnx(:,:,ii)=lyapunov_symm(T,R(:,ii)*Qmat(ii,ii)*Qmat(ii,ii)*R(:,ii)');
    end
end  
%=================
%Storage Matrices 
%=================
vstar=zeros(nz,nobs); 
finmat=zeros(nz,nz,nobs); 

kpartg=zeros(ny,nz,nobs); 
logLnc=zeros(nobs,1); 
yfor=zeros(nz,nobs);
% Matrices with one additional entry (initialization)
stt=zeros(ny,nobs+1);  
ptt=zeros(ny,ny,nobs+1); 
stt(:,1)=azero; 
Pzero   =pshat; 
ptt(:,:,1)=pshat; 
% Start Forward Filter 
cons=nz*log(2*pi); 
if flag_tv == 1;  
    for ii=1:nobs;
        yfor(:,ii)=Z*T*stt(:,ii); 
        [stt(:,ii+1),ptt(:,:,ii+1),logLnc(ii),vstar(:,ii),finmat(:,:,ii),kpartg(:,:,ii)]...
             =kf_dk(y(:,ii),Z,stt(:,ii),ptt(:,:,ii),T,R*diag(Qmat(ii,:)));
    end 
else
    RQ=R*Qmat'; 
    for ii=1:nobs;
        yfor(:,ii)=Z*T*stt(:,ii);
        [stt(:,ii+1),ptt(:,:,ii+1),logLnc(ii),vstar(:,ii),finmat(:,:,ii),...
             kpartg(:,:,ii)]=kf_dk(y(:,ii),Z,stt(:,ii),ptt(:,:,ii),T,RQ);
    end 
end;
logLnc=-0.5*nobs*nz*log(2*pi)+sum(logLnc);
% Truncate First Observation which is the initialization 
stt  =stt(:,2:end); 
%Pzero=ptt(:,:,1); 
ptt=ptt(:,:,2:end); 
yfor=yfor'; 
% Obtain the Innovation using a disturbance smoother 
% Use same recursion to obtain STATE smoother 
etamat=zeros(nx,nobs); 
smooth_st=zeros(ny,nobs); 
Ztr=Z';
Rtr=R'; clear R; 
Ttr=T'; 
% ======================
% Smooth the disturbances 
% =======================
rstar=zeros(ny,1);
rmat=zeros(ny,nobs);
switch flag_tv
    case 0;
        Q=Qmat'*Qmat;
        % First smooth state
        [rstar,etamat(:,end)]=smoothdis(rstar,Q,Rtr,Ztr,finmat(:,:,end),zeros(ny),vstar(:,end));
        smooth_st(:,ii)=stt(:,end)+ptt(:,:,end)*rstar;
        rmat(:,end)=rstar;
        for ii=(nobs-1):-1:1;
            [rstar,etamat(:,ii)]=smoothdis(rstar,Q,Rtr,...
                Ztr,finmat(:,:,ii),((T-T*kpartg(:,:,ii)*Z)'),vstar(:,ii));
            smooth_st(:,ii)=stt(:,ii)+ptt(:,:,ii)*rstar;
            rmat(:,ii)=rstar;
        end
    case 1;
        for ii=nobs:-1:1;
            %Ltr=Ttr-Ztr*kmat(:,:,ii)';
            J=diag( Qmat(ii,:).*Qmat(ii,:) );
            [rstar,etamat(:,ii)] = smoothdis( rstar, J, Rtr, Ztr, finmat(:,:,ii),((T-T*kpartg(:,:,ii)*Z)'), vstar(:,ii) );
            rmat(:,ii) = rstar;
            smooth_st(:,ii)=stt(:,ii)+ptt(:,:,ii)*rstar;
        end
end
etamat=etamat';
smooth_st=smooth_st';
% =======================================================
% Smooth the states using a DISTURBANCE 
% as opposed to a STATE SMOOTHER 
% ========================================================
% Use the last rstar for forward state smoothing 
smooth_st2=zeros(ny,nobs); 
outcount.aold=azero;outcount.Pold=Pzero; 
aold=azero; Pold=Pzero; 
smooth_st2(:,1)=T*aold+Pold*rstar; 
% This decomposition can only be meaningfully done if the initial 
% dispersion is NOT altered 
if flag_instate==0
    countszero=zeros(ss,nx);
    for ii=1:nx
        countszero(:,ii)=pnx(:,:,ii)*rstar;
    end
    % ==========================================================
    % Check can decompose initial state and also that recover the
    % observables
    % ===========================================================
    if max(max(abs( sum(countszero,2)-smooth_st2(:,1) ))) > 1e-4
        warning('Initial States cannot be decomposed across shocks')
    end
else 
    disp('Initial decomposition not feasible when altering initial dispersions') 
    countszero=[]; 
end 
% Observables 
ycheck=zeros(size(y)); 
ycheck(:,1)=Z*smooth_st2(:,1); 
R=Rtr'; 
for ii=2:nobs; 
    smooth_st2(:,ii)=T*smooth_st2(:,ii-1) + R*(etamat(ii,:)'); 
    ycheck(:,ii)=Z*smooth_st2(:,ii); 
end 
ydiscrep=max((abs(ycheck'-y'))); 
if max(ydiscrep) > 1e-4; 
    warning('Smooth estimates cannot recover data');
    disp('WARNING: Smooth estimates could not recover data')
    disp(['Problems for column: ', num2str(find(max(ydiscrep))>1e-4)]); 
    quer('c');
end
dispaj('Maximum discrepancy observables and smoothed: ',num2str(max(ydiscrep))); 
smooth_st2=smooth_st2'; %Checked this matches SMOOTH_ST, Oct 23 07
if max(max(abs( smooth_st2-smooth_st ))) > 1e-4 
    %error('SMOOTH_ST and SMOOTH_ST2 do not match') 
    quer('c'); 
end 
vstar=vstar'; 
if flag_instate==1 && ~isempty(ins.state_data);
    smooth_st = [zeros(1,ny);smooth_st]; 
    stt       = [zeros(1,ny); stt ]; 
    etamat    = [zeros(1,nx);etamat]; 
    yfor      = [zeros(1,nz);yfor]; 
    vstar     = [zeros(1,nz);vstar]; 
end 