function [irfs, irfsboot,  u1, psi, VARirfs, VARirfsboot,alpha] = PROXYSVARRecoverabilityCase(y,z,p,nlagsimp,r,cumind,nrepli,refvar,refhor,m,x)

% computes the impulse response functions and the structural shock in case of
% recoverability
% inputs: y = VAR data; z = proxy; p = number of lags in the VAR; r =
% number of leads used in the projection of z onto the VAR resuduals;
% cumind = indeces of the variables whose ITFs should be cumulated; nrepli
% = number of bootstraps; refvar,refhor = reference variable and horizon for the sign of the IRFs
% m <= p : no. of lags in proxy cleaning: 0 = no cleaning; (default = p)
% x = regressors for the proxy cleaning; (default = y)
%  (the new proxy is the residual of the regression of 
% the original proxy onto its own past and the past of x) 

%% Preliminary 
if nargin < 11
    x = y;
end
if nargin < 10
    m = p;
end
if nargin < 9
    refhor = 1;
end
if nargin < 8
    refvar = 1;
end
if nargin < 7
    nrepli = 500;
end
if nargin < 6
    cumind =[];
end
if nargin < 5
    r = 1;
end
if nargin < 4
    nlagsimp = 32;
end

[T,n] = size(y);
%% Cleaning the instrument z

if m > 0

a = find(~isnan(z));
Z = z(a,:);
yy = x(a,:);
Tyy = size(yy,1);
YL = [ones(Tyy,1) lagmatrix(yy,1:m)];
ZL = lagmatrix(Z,1:m);
X = [YL(m+1:end,:) ZL(m+1:end,:)];
Z = Z(m+1:end,:);
zpar = (X'*X)\X'*Z;
zz = z(p+1:end,:);
zzz = Z - X*zpar; 
zzz = zzz(p-m+1:end,:);
l = length(zzz);
zz(end-l+1:end,:) = zzz;
else
    zz = z(p+1:end,:);
end




%% point estimate

[irfs, u1, par,  delta, epsilon,  v , VARirfs,alpha] = pointestimate(y,zz,p,nlagsimp,r);
u1 = [zeros(p,1)*nan; u1;zeros(r-1,1)*nan];
%%  bootstrap

Tp = T-p;
yboot = y;
Xboot = ones(Tp,n*p+1);
zzboot = zz;
shboot = epsilon; 

  for i = 1:nrepli

      %% constructing artificial data

       draw = ceil(rand(Tp-r+1,1)*(Tp-r+1));
     
      shboot(1:end-r+1,:) = epsilon(draw,:);
      eps = [ones(Tp,1) lagmatrix(shboot, -(0:r-1))];
      eps(end-r+2:end,:) = [];
      vv = v(draw);
      vv(isnan(vv)) = 0;
      zzboot(1:end-r+1,:) = eps*delta + vv;

      for t = p+1:T
           
          for k = 1:p
             Xboot(t-p,(k-1)*n+2:k*n+1)  =  yboot(t-k,:);
            
          end
         yboot(t,:) = Xboot(t-p,:)*par + shboot(t-p,:);

      end

      %% estimating artificial data
   
      [irfsboot(:,:,:,i),~,~,~,~,~,VARirfsboot(:,:,:,i)] = pointestimate(yboot,zzboot,p,nlagsimp,r);

  end

  irfs(cumind,:,:) = cumsum(irfs(cumind,:,:),3);
  irfsboot(cumind,:,:,:) = cumsum(irfsboot(cumind,:,:,:),3); 
  VARirfs(cumind,:,:) = cumsum(VARirfs(cumind,:,:),3);
  VARirfsboot(cumind,:,:,:) = cumsum(VARirfsboot(cumind,:,:,:),3); 

%% point estimate function

function [irfs, u1, par, delta, epsilon,  v, VARirfs,alpha] = pointestimate(y,z,p,nlagsimp,r)

[T,n] = size(y);


Y = y(p+1:end,:);
X = [ones(T,1) lagmatrix(y,1:p)];
X = X(p+1:end,:);
par = (X'*X)\X'*Y;
epsilon = Y-X*par;
Tp = T-p;
Sig = cov(epsilon);
CCC= chol(Sig,'lower');
np=n*p;
par = par';
A(:,:,1) = eye(n);
A(:,:,2:p+1) = - reshape(par(:,2:np+1),n,n,p);
w = invertepolynomialmatrix(A,nlagsimp);
for k = 1:nlagsimp
VARirfs(:,:,k) = w(:,:,k)*CCC;
end
par = par';     
%% estimating psi(L) 

a = find(~isnan(z));
ZZ = z(a);
%TT = length(z);

Y = epsilon(a,:);
X = [ones(size(Y,1),1) lagmatrix(ZZ, (0:r-1))];
X = X(r+1:end,:);
Y = Y(r+1:end,:);
psi = (X'*X)\X'*Y;

%% estimating u1 
X = [ones(Tp,1) lagmatrix(epsilon, -(0:r-1))];
X(end-r+2:end,:) = [];
Y = z(1:end-r+1);
a = find(~isnan(Y));
XX = X(a,:); 
YY = Y(a,:);
delta = (X'*X)\XX'*YY;
fitted = X*delta; 
v = Y - fitted;
%alpha = std(fitted);
u1 = fitted/std(fitted);

%% computing IRfs
for ii = 1 : r
        B(:,1,ii) = psi(ii+1,:)';
        BB(ii) = (B(:,1,ii)'/Sig)*B(:,1,ii);
        %u1 = u1+ (epsilon(ii:end-r+ii,:)/Sig)*B(:,1,ii);
end
alpha = cov(ZZ)*sqrt(sum(BB));
irfs = polynomialmatricesproduct(w,B,nlagsimp)/sqrt(sum(BB));
irfs = irfs*sign(irfs(refvar,1,refhor));
end

end