% BCL_Sign - ## Barigozzi, Conti & Luciani with Sign Restrictions (Main file)
% 
% Replication file for the paper:
% "Do Euro Area Countries Respond Asymmetrically to the Common Monetary Policy?,"
% by Matteo Barigozzi, Antonio M. Conti, and Matteo Luciani
% to appear on the Oxford Bulletin of Economics and Statistics
% available online since June 23, 2013. 
% 

%  written by Matteo Luciani - mluciani@ulb.ac.be

clear all; close all; clc
TR=2; out=2;                                                                % Set Data Transformation
[DB, label, tt, cd] = ML_ReadDataEU(TR,out);                                % Upload Data - Trnsformed Series
[RawDB, ~, TT] = ML_ReadDataEU(0,out);                                      % Upload Data - Non Trnsformed Series
DB(end-3:end,:)=[]; tt(end-3:end)=[];                                       % cut the sample, it eliminates data for 2008
[T N]=size(DB);                                                             % Usefull Objects
[y,~,sigma]=ML_Standardize(DB);                                             % Standardize Data and store the standar deviation 
ML_graph_options                                                            % Set graphic options
r=12; p=2;  q=4; s=21; det=1;                                               % Set-up Model Parameters
draws=50000;                                                                % number of draws for the rotation matrix
boot=500;                                                                   % bootstrap draws
maxrot=10;                                                                  % maximum number of rotations
ms=1;                                                                       % identify the position of the MP shock
h=3;                                                                        % horizons on which the restrictions must hold
tb1=[63 1]; tb2=[63 2];                                                     % Break point for 1999
EU=[228:231 233:236];                                                       % Identifies EA variables for identification
nrest=ones(q,1)*230;                                                        % Normalization Restrictions
for kk=1:h; rest{kk}=[EU([1 2 5 3 7 8]); -ones(1,3) ones(1,3)]'; end;       % Sign Restrictions

[Imp,~,~,imp]=ML_DfmSignImp(DB,cd,q,r,p,s,nrest,rest,draws,h,100,1);        % Impulse responses whole sample
[Imp1,~,~,imp1]=ML_DfmSignImp(DB,cd,q,r,p,s,nrest,rest,draws,h,100,1,tb1);  % Impulse responses before 1998
[Imp2,~,~,imp2]=ML_DfmSignImp(DB,cd,q,r,p,s,nrest,rest,draws,h,100,1,tb2);  % Impulse responses after 1998
[CC0,CC1,CC2]=BCL_SignBoot2(DB,cd,q,r,p,21,nrest,rest,draws,h,boot,maxrot); % Bootstrap Confidence band

CC0 = cat(4,CC0,Imp); CC1 = cat(4,CC1,Imp1);  CC2 = cat(4,CC2,Imp2);                    % Merge the Bootstrap and the sample IRF
for jj=1: size(CC0,4); IRF0(:,:,jj)=ML_IRFnormalize(squeeze(CC0(:,1,:,jj))',.5,230); end   % Standardize all IRF obtained with Bootstrap
for jj=1: size(CC1,4); IRF1(:,:,jj)=ML_IRFnormalize(squeeze(CC1(:,1,:,jj))',.5,230); end   % Standardize all IRF obtained with Bootstrap
for jj=1: size(CC2,4); IRF2(:,:,jj)=ML_IRFnormalize(squeeze(CC2(:,1,:,jj))',.5,230); end   % Standardize all IRF obtained with Bootstrap
IRF0u=prctile(IRF0,84,3); IRF0d=prctile(IRF0,16,3);  IRF0m=prctile(IRF0,50,3);          % Median and Confidence Band
IRF1u=prctile(IRF1,84,3); IRF1d=prctile(IRF1,16,3);  IRF1m=prctile(IRF1,50,3);          % Median and Confidence Band
IRF2u=prctile(IRF2,84,3); IRF2d=prctile(IRF2,16,3);  IRF2m=prctile(IRF2,50,3);          % Median and Confidence Band


% ________________________________________________ %
% Grouping Variables: Useful for Graphs and Tables %
%  %
Reddito=[1 32 66 100 134 167 199 203 206 212 228];  
Prezzi=Reddito+[22 23 23 23 22 21  1   1   1   1   1];
Consumo=Reddito(1:6)+1;
Investimento=Reddito(1:6)+2;
Disoccupazione=[ 1 32 66 100 134 167]+[13 14 14  14  13  13];
EU=[228:231 233:236];

% Reshape Grouped variables
Reddito=Reddito([3 2 5 1 7 4 6 10 9 8]);
Prezzi=Prezzi([3 2 5 1 7 4 6 10 9 8]);
Consumo=Consumo([3 2 5 1 4 6]);
Investimento=Investimento([3 2 5 1 4 6]);
Disoccupazione=Disoccupazione([3 2 5 1 4 6]);


% _________ %
% Labelling %
%  %
EUlabel{1}='GDP'; EUlabel{2}='CPI'; EUlabel{5}='M1'; EUlabel{6}='M3';
EUlabel{3}='Short Term Interest Rate'; EUlabel{4}='Long Term Interest Rate';
EUlabel{7}='Real Effective Exchange Rate'; EUlabel{8}='Dollar/Euro Exchange Rate';
country{1}='Germany'; country{2}='France'; country{3}='Netherlands';
country{4}='Belgium'; country{5}='Finland'; country{6}='Italy';        
country{7}='Spain'; country{8}='Portugal'; country{9}='Ireland';
country{10}='Greece';
clc

%%% =================== %%%
%%% Testing Asymmetries %%%
%%% =================== %%%
for jj=1:5;     
    if     jj==1; idvar=[Reddito 228]; J=11;
    elseif jj==2; idvar=[Prezzi 229]; J=11;
    elseif jj==3; idvar=Disoccupazione; J=1;
    elseif jj==4; idvar=Consumo; J=1;
    elseif jj==5; idvar=Investimento; J=1;
    end
    [D1m{jj},D1u{jj},D1d{jj}]=BCL_TestingDistances(IRF1(:,idvar,:),J);
    [D2m{jj},D2u{jj},D2d{jj}]=BCL_TestingDistances(IRF2(:,idvar,:),J);
end

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%% ====================== %%%
                %%% === -------------- === %%%
                %%% === -- Graphing -- === %%%
                %%% === -------------- === %%%
                %%% ====================== %%%
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%% =================================== %%%
%%% Impulse Responses for EU Aggregates %%%
%%% =================================== %%%
set(0,'defaultaxesfontsize',8);
figure(2); ML_FigureSize; idvar=EU([1 2 5 3 7 8]); 
disp('Figure 2. Impulse responses to a monetary policy shock: Euro Area aggregates');
for ii=1:6;
    subplot(3,2,ii), hold on; 
    ha=area(0:20,[IRF1d(1:21,idvar(ii)) IRF1u(1:21,idvar(ii))-IRF1d(1:21,idvar(ii))],'linestyle','none');
    set(ha(1), 'FaceColor', 'none'); set(ha(2), 'FaceColor', [0.7 0.7 0.7]) 
    plot(0:s-1,IRF1m(:,idvar(ii)),'linewidth',2,'Color',[0.5 0.5 0.5]); 
    plot(0:s-1,zeros(s,1),'k');                                                       
    plot(0:s-1,IRF2u(:,idvar(ii)),'k--',0:s-1,IRF2d(:,idvar(ii)),'k--','linewidth',2);
    plot(0:s-1,IRF2m(:,idvar(ii)),'k-','linewidth',2); 
    hold off; axis tight         
    title(EUlabel{ii},'fontangle','italic','fontsize',10);    
end
              

%%% ============================== %%%
%%% Impulse Responses by Countries %%%
%%% ============================== %%%

% GDP and CPI %
set(0,'defaultaxesfontsize',6); 
for jj=1:2;     
    if jj==2; idvar=Reddito;	
        disp('Figure 4. Impulse responses to a monetary policy shock: Gross Domestic Product');
    elseif jj==1; idvar=Prezzi;  
        disp('Figure 3. Impulse responses to a monetary policy shock: Consumer Price Index'); 
    end
       
    y1=max(max(max([IRF1u(:,idvar) IRF2u(:,idvar)])));              % These lines are needed to construct
    y2=min(min(min([IRF1d(:,idvar) IRF2d(:,idvar)])));              % boxes all equal
    
    figure(2+jj),ML_FigureSize;
    for ii=1:max(size(idvar));
        subplot(3,5,ii), hold on;
        ha=area(0:20,[IRF1d(1:21,idvar(ii)) IRF1u(1:21,idvar(ii))-IRF1d(1:21,idvar(ii))],'linestyle','none');
        set(ha(1), 'FaceColor', 'none'); set(ha(2), 'FaceColor', [0.7 0.7 0.7]);
        plot(0:s-1,IRF1m(:,idvar(ii)),'linewidth',1.5,'Color',[0.5 0.5 0.5]); 
        plot(0:s-1,zeros(s,1),'k'); 
        plot(0:s-1,IRF2u(:,idvar(ii)),'k--',0:s-1,IRF2d(:,idvar(ii)),'k--','linewidth',1.5); 
        plot(0:s-1,IRF2m(:,idvar(ii)),'k-','linewidth',1.5); 
        hold off; axis([0 s-1 y2 y1]); title(country{ii},'fontangle','italic','fontsize',10);
    end;
end
 

% Consumption, INvestment & Unemployment Rate %
jj=0; for ii=[1:4 6 7]; jj=jj+1; country1{jj}=country{ii}; end

for jj=3:7;     
    if jj==5; idvar=Disoccupazione;   
        disp('Figure 7. Impulse responses to a monetary policy shock: Unemployment Rate'); 
    elseif jj==3; idvar=Consumo;        
        disp('Figure 5. Impulse responses to a monetary policy shock: Consumption'); 
    elseif jj==4; idvar=Investimento;     
        disp('Figure 6. Impulse responses to a monetary policy shock: Investment'); 
    end    
    y1=max(max(max([IRF1u(:,idvar) IRF2u(:,idvar)])));              % These lines are needed to construct
    y2=min(min(min([IRF1d(:,idvar) IRF2d(:,idvar)])));              % boxes all equal
    figure(2+jj),ML_FigureSize
    for ii=1:max(size(idvar));                    
        subplot(3,3,ii),    hold on;
        ha=area(0:20,[IRF1d(1:21,idvar(ii)) IRF1u(1:21,idvar(ii))-IRF1d(1:21,idvar(ii))],'linestyle','none');
        set(ha(1), 'FaceColor', 'none'); set(ha(2), 'FaceColor', [0.7 0.7 0.7]) 
        plot(0:s-1,IRF1m(:,idvar(ii)),'linewidth',1.5,'Color',[0.5 0.5 0.5]); 
        plot(0:s-1,zeros(s,1),'k');
        plot(0:s-1,IRF2u(:,idvar(ii)),'k--',0:s-1,IRF2d(:,idvar(ii)),'k--','linewidth',1.5); 
        plot(0:s-1,IRF2m(:,idvar(ii)),'k-','linewidth',1.5);
        hold off; axis([0 s-1 y2 y1]); title(country1{ii},'fontangle','italic','fontsize',12);
    end; ii=1;
end


%%% =================================================== %%%
%%% Testing Asymmetries Distance from Reference Country %%%
%%% =================================================== %%%

for jj=1:5;
    if     jj==1; J=11; titolo=country;  name='Quantifying asymmetries: distance from benchmark country: GDP';
    elseif jj==2; J=11; titolo=country;  name='Quantifying asymmetries: distance from benchmark country: CPI';
    elseif jj==3; J=1;  titolo=country1; name='Quantifying asymmetries: distance from benchmark country: UR';
    elseif jj==4; J=1;  titolo=country1; name='Quantifying asymmetries: distance from benchmark country: Cons';
    elseif jj==5; J=1;  titolo=country1; name='Quantifying asymmetries: distance from benchmark country: Inv';
    end      
    y1=ML_min([D1u{jj} D2u{jj}],2);               % These lines are needed to construct
    y2=ML_min([D1d{jj} D2d{jj}]);                 % boxes all equal    
    figure(7+jj); ML_FigureSize; kk=0;
    disp(['Figure ' num2str(7+jj) '. ' name]) 
    for ii=1:size(D2m{jj},2);
        if ii~=J; kk=kk+1;
            subplot(3,5,kk), hold on;
            ha=area(0:20,[D1d{jj}(1:21,ii) D1u{jj}(1:21,ii)-D1d{jj}(1:21,ii)],'linestyle','none');
            set(ha(1), 'FaceColor', 'none'); set(ha(2), 'FaceColor', [0.7 0.7 0.7]);            
            plot(0:s-1,D1m{jj}(:,ii),'linewidth',1.5,'Color',[0.5 0.5 0.5]);
            plot(0:s-1,zeros(s,1),'k');
            plot(0:s-1,D2u{jj}(:,ii),'k--',0:s-1,D2d{jj}(:,ii),'k--','linewidth',1.5);
            plot(0:s-1,D2m{jj}(:,ii),'k-','linewidth',1.5);            
            hold off; title(titolo{ii},'fontangle','italic','fontsize',10);
            axis tight
            axis([0 s-1 y2 y1]);
        end
    end;    
end

