% ML_factor_loadings_break - Estimates lambda, chi and xsi components when there are breaks in the factor loadings
%
% function [lambda, chat, xsihat, LAMBDA]=ML_factor_loadings_break(X,fhat,Tbreak)
%
% Inputs
%   X      - data matrix
%   fhat   - factors estimated over the whole sample
%   Tbreak - vector containing the break points
% Outputs
%   lambda - factors loadings for the last time span
%   chat   - common components
%   xsihat - idiosyncratic companent
%   LAMBDA - N by r by nb+1 matrix containing the loading for each subperiod
%

%  written by Matteo Luciani - mluciani@ulb.ac.be

function [lambda, chat, xsihat, LAMBDA]=ML_factor_loadings_break(X,fhat,Tbreak)

N=size(X,2);
[t r]=size(fhat);
nb=size(Tbreak,2);

xx=[fhat];                        
xy=[fhat];   
for ii=1:nb;    
    F=[zeros(Tbreak(ii)-1,r); fhat(Tbreak(ii):t,:)];
    xx=[xx F];   
end

for i=1:N; [beta(:,i),xsihat(:,i)]=ML_ols(X(:,i),xx,0); chat(:,i)=X(:,i)-xsihat(:,i); end;  % Estimating Common & idiosyncratic components
for i=1:r; temp(i,:)=beta(i,:); for jj=1:nb; temp(i,:)=temp(i,:)+(beta(i+jj*r,:)); end; end;% Estimating Factors loadings
lambda=temp';

lambda_tilda=beta(1:r,:)';
for jj=1:nb+1;
    Beta(:,:,jj)=beta(1+(jj-1)*r:jj*r,:); 
    if jj==1;BETA(:,:,jj)=Beta(:,:,jj);
    else BETA(:,:,jj)=BETA(:,:,jj-1)+Beta(:,:,jj); end
    LAMBDA(:,:,jj)=BETA(:,:,jj)';    
end;