% BLL_ReplicationFile1 - IRF for Monetary policy shock

clear all; close all; clc; 
ML_graph_options
sel=2; trans=2;
r=7; q=3; d=2; s=100; p=2; det=1;                                           % parameter set up
xi=68; up=.5*(100+xi); down=.5*(100-xi);                                    % set confidence level
[Y, Label, Name, Dates, cd] = BLL_ReadData(3,1,sel);                        % Load data in levels
[Data,~,~,~, codeData] = BLL_ReadData(2,1,sel);                             % Load Data in first differences
[T,N]=size(Y);                                                              % size of the panel
J=BL_TestLinearTrend(ML_diff(Y));                                           % Test for Linear trend
J1=[34 43 73 74 75 76 77 78 87];                                            % Variables for which I want an overrule
J2=find(~ismember(1:length(J),J1));                                         % variables for which I want to accept the test
J3=J; J3(J1)=1;                                                             % Customized detrending
J=J3;

%%% ======================================== %%%
%%%  Set Parameters for Structural Analysis  %%%
%%% ======================================== %%%
boot=10;                                                                 	% number of bootstrap
idvar=[31 7 65];                                                            % variables on which restrictions are imposed
LRR=0; SRR=[1 2;1 3;2 3];                                                   % choleski
cd(7)=0; codeData(7)=1;                                                     % this is to show IRF for Inflation

%%% ======================= %%%
%%%  Structural Estimation  %%%
%%% ======================= %%%
[C{1},D1,sh,] = ML_DfmSLRImp(Data,codeData,q,r,p-1,s,idvar,SRR,LRR);        % IRF - Black Box
[CC{1}, eflag1] = ML_SLRBoot2(Data,codeData,q,r,p-1,s,idvar,SRR,LRR,boot);  % Bands - Black Box
[C{2},e] = BLL_SLRImp(Y,cd,q,d,r,p,s,idvar,SRR,LRR,J);                      % IRF - BLL (VECM)
[CC{2}, eflag2] = BLL_SLRBoot2(Y,cd,q,d,r,p,s,idvar,SRR,LRR,boot,J);        % Bands - BLL (VECM)
[C{3},sh2] = BLL_SLRImpVAR(Y,cd,q,r,p,s,idvar,SRR,LRR,J);                   % IRF - BLL (VAR)
[CC{3}, eflag3] = BLL_SLRBootVAR2(Y,cd,q,r,p,s,idvar,SRR,LRR,boot,0,J);     % Bands - BLL (VAR)

%%% ==================================================== %%%
%%%  Rearrange and Normalize Impulse Response Functions  %%%
%%% ==================================================== %%%
qq=3;                                                                       % Keep only third shock, i.e. MP shock
for mm=1:3; 
    IRF{mm}=squeeze(C{mm}(:,qq,:))';
    IRF{mm}=ML_IRFnormalize(IRF{mm},.5,65);
    IRFboot{mm}=NaN(s,N,boot);                                              % preallocate       
    for bb=1:boot;
        IRFboot{mm}(:,:,bb)=squeeze(CC{mm}(:,qq,:,bb))';
        IRFboot{mm}(:,:,bb)=ML_IRFnormalize(IRFboot{mm}(:,:,bb),.5,65);
    end
end

%%% ================== %%%
%%%  Confidence bands  %%%
%%% ================== %%%
for mm=1:3;    
    first=mean(IRFboot{mm},3)';
    second=mean(IRFboot{mm}.^2,3)';
    stde=sqrt(second-first.^2)';
    IRFu{mm,1} = IRF{mm} + stde;
    IRFd{mm,1} = IRF{mm} - stde;
end

        %%%%% =========================== %%%%%
        %%%%% ==== ----------------- ==== %%%%%
        %%%%% ==== ---   Graphs  --- ==== %%%%%
        %%%%% ==== ----------------- ==== %%%%%
        %%%%% =========================== %%%%%

ID=[31 7 65 101];                                                     % Variables of Interests
Variable={'Gross Domestic Product','CPI inflation',...
    'Federal Funds Rate','S&P 500 Stock Price Index'};
Modello={'VAR in differences','VECM','Unrestricted VAR in levels'};
s1=81;

%%% ===================================== %%%
%%%  All variables all models one by one  %%%
%%% ===================================== %%%

s2=81;
for ii=1:4;
    a=min(0,ML_min([IRFd{3,1}(1:s2,ID(ii)) IRFd{2,1}(1:s2,ID(ii)) IRFd{1,1}(1:s2,ID(ii))])); %
    b=max(0,ML_min([IRFu{3,1}(1:s2,ID(ii)) IRFu{2,1}(1:s2,ID(ii)) IRFu{1,1}(1:s2,ID(ii))],2)); %
    c=round(100*(b-a)/10)/100; 
    for mm=1:3;
        axes('Parent',figure,'FontSize',14);% ML_FigureSize(1); 
        hold on;        
        plot(0:s2-1,zeros(s2,1),'k');        
        plot(0:s2-1,[IRFu{mm,1}(1:s2,ID(ii)) IRFd{mm,1}(1:s2,ID(ii))],'k--','linewidth',2);
        plot(0:s2-1,IRF{mm}(1:s2,ID(ii)),'k-','linewidth',3);
        hold off; axis tight; box on
        set(gca,'Xtick',0:8:s2,'Xticklabel',0:2:20,'Layer','top')
        ytick=ML_ytick(a,b,c);
        set(gca,'Ytick',ytick,'Yticklabel',num2str(ytick','%.2f'))
        axis([xlim+eps a-c/10 b+c/10])
        xlabel('Years After the shock','fontangle','italic','fontsize',12);
        ylabel(Modello{mm},'fontangle','italic','fontsize',12);
        title(Variable{ii},'fontangle','italic','fontsize',18);
    end
end
