% BL_TestLinearTrend - test if there is a linear trend
% 
% In order to choose whether or not to de-trend a variable we apply the following procedure: 
% 	let $m_i$ be the sample mean of $\Delta y_{it}$, 
% 	$\gamma_{i}(j)$ be the auto-covariance of order $j$ of $\Delta y_{it}$, 
% 	and $\bar{\gamma}_i=\sqrt{\frac{1}{T}\sum_{j=1}^J \gamma_i(j)}$, 
% 	then if  $\frac{|m_i|}{\bar{\gamma}_i}\ge 1.96$ we estimate $a_i$ and $b_i$ 
% 	from an OLS regression of $y_{it}$ on a constant and a time trend, 
% 	whereas if $\frac{|m_i|}{\bar{\gamma}_i}< 1.96$ we set $\wh{a}_i=m_i$ and $\wh{b}_i=0$.
% 



function J=BL_TestLinearTrend(y)
[T,N]=size(y);
my=mean(y);


for ii=1:N;    
    for jj=0:15;
        x1=y(jj+1:end,ii)-my(ii);
        x2=y(1:end-jj,ii)-my(ii);
        autocov(jj+1,ii)=(x1'*x2)/(T-jj-1); % autocovariance
        clear x1 x2
    end    
end;

sm=sqrt(sum(autocov)/T);
J=(abs(my)./sm)>=1.96;